/*
 * log.c -- logging and error handling
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>
#include <sys/types.h>
#include <unistd.h>

#include "common.h"
#include "log.h"


static FILE *logFile;


static char *dateAndTimeNow(void) {
  long now;

  now = time(NULL);
  return ctime(&now);
}


void logOpen(char *logName) {
  logFile = fopen(logName, "a");
  if (logFile == NULL) {
    fprintf(stderr, "Error: cannot open log file '%s'\n", logName);
    exit(1);
  }
  fprintf(logFile, "Process %d, %s", getpid(), dateAndTimeNow());
  fprintf(logFile, "-- Start logging --\n");
  fflush(logFile);
}


void logClose(void) {
  fprintf(logFile, "Process %d, %s", getpid(), dateAndTimeNow());
  fprintf(logFile, "-- Stop logging --\n");
  fflush(logFile);
  fclose(logFile);
}


void logMessage(char *fmt, ...) {
  va_list ap;

  va_start(ap, fmt);
  fprintf(logFile, "Process %d, %s", getpid(), dateAndTimeNow());
  vfprintf(logFile, fmt, ap);
  fflush(logFile);
  va_end(ap);
}


void logPanic(char *fmt, ...) {
  va_list ap;

  va_start(ap, fmt);
  fprintf(logFile, "Process %d, %s", getpid(), dateAndTimeNow());
  fprintf(logFile, "Panic: ");
  vfprintf(logFile, fmt, ap);
  fprintf(logFile, "\n");
  fflush(logFile);
  va_end(ap);
  logClose();
  exit(1);
}
